------------------------------------------------------------------------
-- max2.vhd			Steven Sutankayo
--                              Duncan Elliott
-- July 14, 1998
-- Test file for Altera UP1 Board
-- 
-- Tests XTAL oscillator, stuck-at faults on all general I/O pins, 
-- and both 7-segment displays.
------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity max2 is
        port(
                gclk1 : in std_logic;       -- dedicated inputs

				iosig : inout std_logic_vector(1 to 60)
				
--				sev_seg1, sev_seg2 : out std_logic_vector(1 to 8)
        );
end;

architecture mixed of max2 is
	signal 	halved_crystal, slow_crystal : std_logic;
	signal  input_pattern : std_logic_vector(1 to 3);
		
	signal 	clock_count	: std_logic_vector(23 downto 0);

begin

	---------------------------------------------
	-- clock divider process 
	-- halves the clock so that we don't run into
	-- timing problems with the clock counter
	---------------------------------------------
	clock_divider : process (gclk1) is
	begin
		if gclk1'event and gclk1 = '1' then
			if halved_crystal = '1' then
				halved_crystal <= '0';
			else
				halved_crystal <= '1';
			end if;
		end if;
	end process clock_divider;

	---------------------------------------------
	-- divides the halved clock by 4 million
	-- ( so 8 million in total )
	----------------------------------------------
	clock_counter : process (halved_crystal) is
	begin
		if halved_crystal'event and halved_crystal = '1' then

			-- check if we have counted to 2e8480
			if clock_count = x"400000" then
--													if clock_count = x"000080" then  -- (for simulation)
				clock_count <= x"000000";

				-- toggle the slow crystal	
				if slow_crystal = '1' then
					slow_crystal <= '0';
				else
					slow_crystal <= '1';
				end if;
			else 
			   clock_count <= clock_count + x"000001";
			end if;
			
		end if;	
	end process clock_counter;
	
	pattern_generator : process( gclk1 ) is
		variable pattern_count : std_logic_vector( 3 downto 0 ) := x"0";
	begin
		if rising_edge(slow_crystal) then
			if pattern_count = x"5" then
				pattern_count := x"0";
			else
				pattern_count := pattern_count + x"1";
			end if;
		end if;
		
		case pattern_count is 
			when x"0" =>
				input_pattern <= "100";
			when x"1" =>
				input_pattern <= "110";
			when x"2" =>
				input_pattern <= "010";
			when x"3" =>
				input_pattern <= "011";
			when x"4" =>
				input_pattern <= "001";
			when x"5" =>
				input_pattern <= "101";
			when others =>
				input_pattern <= "100";
		end case;
	end process pattern_generator;
	
			
	----------------------------------------------
	-- pin assignments
	-- the general purpose i/o pins are assigned 
	-- to each other sequentially so that faults 
	-- can be detected.
	----------------------------------------------
	iosig(1) <= input_pattern(1);
	iosig(2) <= input_pattern(2);
	iosig(3) <= input_pattern(3);
		
	assign : for i in 2 to 20 generate
	begin
		iosig(3*i-2) <= iosig(3*(i-1)-2);
		iosig(3*i-1) <= iosig(3*(i-1)-1);
		iosig(3*i)   <= iosig(3*(i-1));
	end generate;
	
end mixed;

